#include <iostream>
#include <fstream>
#include <cstdlib>
#include <vector>
#include <cmath>
#include "TMDlib_UnintegratedGluonKS.h"
using namespace std;


//-----------------------------------------------------------------------------
UnintegratedGluon::UnintegratedGluon() { }
UnintegratedGluon::UnintegratedGluon(string filename, gsl_interp_type int_type)
				     : _int_type(int_type), _some_eps(1e-10) {

  _errorfile = new ofstream("unintegratedgluon.err");

  // determine grid dimensions (in logx and logkt2 directions) 
  // and set gluon to the interpolation of the 2d grid
  _get_2dgrid_dimensions(filename);
  _set_from_2dgrid(filename);

}

//-----------------------------------------------------------------------------
UnintegratedGluon::~UnintegratedGluon () {
  delete _updf; 
  _errorfile->close();
  delete _errorfile;
}

//-----------------------------------------------------------------------------
// function determining the size of the 2d grid
void UnintegratedGluon::_get_2dgrid_dimensions(string filename) {

  ifstream indata;
  indata.open(filename.c_str());
  if(!indata) { cerr << "Error: updf grid file could not be opened" << endl;
     exit(1); }

  // read all lines in the file; search for the turn of the first
  // variable, this gives n1, then continue reading to the the total
  // number of lines
  double x1, x2, f;
  int nlines = 0;
  bool count2 = true;
  vector<double> x1_vec;
  while(indata >> x1 >> x2 >> f) {
    ++nlines;
    if (count2) {
      x1_vec.push_back(x1);
      if (x1_vec.size()>1) {
        if(abs(x1_vec[x1_vec.size()-1]- x1_vec[x1_vec.size()-2])>_some_eps) {
	  count2 = false;
	  continue;
        }
      }
    }
  }

  // set the grid size in the argument variables, check if the grid is
  // a square in 2d  and print its size
  _n2 = x1_vec.size()-1;
  _n1 = nlines/ _n2;
  if (nlines % _n2) {
     cerr << "Error: updf grid is not a square 2d grid! (nlines/n1 = ";
     cerr << nlines << "/" << double(_n2) << " = ";
     cerr << double(nlines)/double(_n2) << ")"<< endl;
     exit(1);
  } 
  //cout << "# gri2d size (" << _n1 << ", " << _n2 << ")" << endl;
  indata.close();
}

//------------------------------------------------------------------------------
// return size of the grid in each dimension
vector<int> UnintegratedGluon::grid_size() const {
  vector<int> size_vec;
  size_vec.push_back(_n1);
  size_vec.push_back(_n2);
  size_vec.push_back(_n3);
  return size_vec;
}

//------------------------------------------------------------------------------
// set gluon from 2d grid as updf
void UnintegratedGluon::_set_from_2dgrid(string filename) {

  // read the grid input file
  ifstream indata;
  indata.open(filename.c_str());
  if(!indata) { cerr << "Error: updf grid file could not be opened" << endl;
     exit(1);}

  // vectors for passing argument values
  vector<double> vec_logx(_n1);
  vector<double> vec_logkt2(_n2);
  vector<double> vec_xg(_n2);

  // 2d array for passing function values at grid points
  boost::multi_array<double, 2> matrix_xg(boost::extents[_n1][_n2]);

  // Read the grid from the file
  for (unsigned i=0;i<_n1;i++) {        // logx
    for (unsigned j=0;j<_n2;j++) {      // logkt2

        double logx, logkt2, xg;
        indata >> logx >> logkt2 >> xg;
        // saving values in vectors could be improved here
        vec_logx[i] = logx;
        vec_logkt2[j] = logkt2;
        matrix_xg[i][j] = xg;
    }
  }
  indata.close();

  _norm = &_norm_F;
  _updf = new Interpolation2D(vec_logx, vec_logkt2,  matrix_xg, _int_type);
}

// KMS and BK-like normalization
double UnintegratedGluon::_norm_F(const vector<double> &x) {
  return 1.0;
}

//------------------------------------------------------------------------------
double UnintegratedGluon::xg(double logx, double logkt2) {
  vector<double> x(3);
  x[0] = logx;
  x[1] = logkt2;

  // take the value of interpolated gluon
  double val = _norm(x)*(_updf->interp(x));
  // cout << val << endl;

  // check if the corresponding x vector is in the domain of the grid
  // used for interpolation, if not set the gluon to zero
  //if (_updf->eval_status() > 0) {
  //  *_errorfile << "Err001: xg(" << x[0] << ", " << x[1];
  //  *_errorfile << ")" << endl;
  //  val=0.0;
  //}
  // check if the gluon is positive, if not, set it to zero
  if (val < 0 ) {
    *_errorfile << "Err002: xg(" << x[0] << ", " << x[1];
    *_errorfile << ") = " << val << endl;
    val=0.0;
  }

  return val;
}

